@@ -53,6 +53,7 @@
 #include "fragmentationDlg.hpp"
 #include "massSearchDlg.hpp"
 #include "sequencePurificationDlg.hpp"
+#include "sequenceImportDlg.hpp"
 #include "mzCalculationDlg.hpp"
 #include "compositionsDlg.hpp"
 #include "pkaPhPi.hpp"
@@ -61,6 +62,7 @@
 #include "crossLink.hpp"
 #include "crossLinkerSpec.hpp"
 #include "decimalPlacesOptionsDlg.hpp"
+#include "monomerDictionary.hpp"
 
 
 namespace massXpert
@@ -82,7 +84,7 @@ namespace massXpert
 				  "sequence editor window."),
 			      QMessageBox::Ok);
       }
-  
+    
     readSettings();
   
     show();
@@ -512,12 +514,22 @@ namespace massXpert
     saveAsAct->setStatusTip(tr("Saves the sequence in another file"));
     connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
   
+
+
     // File/ImportRaw
     importRawAct = new QAction(tr("&Import raw"), this);
     importRawAct->setShortcut(tr("Ctrl+I"));
     importRawAct->setStatusTip(tr("Imports a raw text file"));
     connect(importRawAct, SIGNAL(triggered()), this, SLOT(importRaw()));
   
+    // File/ImportPdb
+    importPdbProtAct = new QAction(tr("&Import PDB"), this);
+    importPdbProtAct->setShortcut(tr("Ctrl+I"));
+    importPdbProtAct->setStatusTip(tr("Imports a PDB file"));
+    connect(importPdbProtAct, SIGNAL(triggered()), this, SLOT(importPdbProt()));
+  
+
+
     // File/ExportClipboard
     exportClipboardAct = new QAction(tr("Export to &clipboard"), this);
     exportClipboardAct->setShortcut 
@@ -680,7 +692,12 @@ namespace massXpert
     fileMenu->addAction(saveAct);
     fileMenu->addAction(saveAsAct);
     fileMenu->addSeparator();
-    fileMenu->addAction(importRawAct);
+
+    // Sequence importers menu
+    fileImportMenu = fileMenu->addMenu(tr("&Import"));
+    fileImportMenu->addAction(importRawAct);
+    fileImportMenu->addAction(importPdbProtAct);
+
     fileMenu->addSeparator();
     fileMenu->addAction(exportClipboardAct);
     fileMenu->addAction(exportFileAct);
@@ -2003,11 +2020,18 @@ namespace massXpert
     while(!stream.atEnd())
       {
 	QString line = stream.readLine(1000);
+	
+// 	qDebug() << __FILE__ << __LINE__
+// 		 << "line:" << line;
+	
 	sequence.appendMonomerText(line);
       }
   
     // And now make sure the sequence is correct.
 
+//     qDebug() << __FILE__ << __LINE__
+// 	     << "Done streaming file." 
+
     QList<int> errorList ;
   
     if (sequence.makeMonomerList(mpa_polymer->polChemDef(),
@@ -2034,7 +2058,347 @@ namespace massXpert
       }
   }
 
+  
+  void
+  SequenceEditorWnd::importPdbProt()
+  {
+    // We want to import the protein sequence data out of a PDB file. 
+    
+    // Format of a line:
+    // SEQRES 1 A 21 GLY ILE VAL GLU GLN CYS CYS THR SER ILE CYS SER LEU
 
+    // PDB File Format - Contents Guide Version 3.20 (Sept 15, 2008)
+    // http://www.wwpdb.org/docs.html
+    // File:
+    // ftp://ftp.wwpdb.org/pub/pdb/doc/format_descriptions/Format_v32_A4.pdf
+    //
+    // The monomer codes are 3 uppercase letters-long.
+
+    // First, get to know what the PDB file is:
+    
+    QString filePath = 
+      QFileDialog::getOpenFileName(this, 
+				   tr("massXpert - Import PDB Protein Sequence"),
+				   QDir::homePath(),
+				   tr("PDB files (*.PDB *.Pdb *.pdb)"));
+    
+    QFile file(filePath);
+    
+    if (!file.exists(filePath))
+      {
+	QMessageBox::critical(this, 
+			      tr("massXpert - Import PDB Protein Sequence"),
+			      tr("File not found."),
+			      QMessageBox::Ok);
+	return;
+      }
+    
+    // Make sure we can read the sequence.
+    
+    if (!file.open(QIODevice::ReadOnly))
+      return;
+    
+    QTextStream stream(&file);
+   
+    // Now, ensure that we can translate the ILE-formatted codes to
+    // the codes we are currently using in our sequence.
+
+    int codeLength = polChemDef()->codeLength();
+
+    // Load the dictionary file "pdb-protein-to-mxp.dic" (from
+    // dataDir/dictionaries) by creating a dictionary object.
+
+    Application *application = static_cast<Application *>(qApp);
+    ConfigSettings *configurationSettings = application->configSettings();
+
+    QString dictionaryFilePath(configurationSettings->systemDataDir() +
+			       QDir::separator() + "dictionaries" + 
+			       QDir::separator() + "pdb-protein-to-mxp.dic");
+    
+    MonomerDictionary dictionary;
+    
+    dictionary.setFilePath(dictionaryFilePath);
+    dictionary.setInputCodeLength(3);
+    dictionary.setOutputCodeLength(codeLength);
+
+    if (!dictionary.loadDictionary())
+      {
+	QMessageBox::critical(this, 
+			      tr("massXpert - Import PDB Protein Sequence"),
+			      tr("Failed to load the dictionary."),
+			      QMessageBox::Ok);
+	return;
+      }
+    
+    // At this point we have loaded the dictionary, let's process the
+    // file contents to extract the protein sequence data out of it.
+    
+    // Create a string list where to store all the chain sequences
+    // parsed in the file and a parallel one where to store the
+    // corresponding chain ID.
+    QStringList *chainStringList = new QStringList();
+    QStringList *chainStringListId = new QStringList();
+    
+    // Create a string to hold the sequence that is going to be
+    // extracted from the PDB file's lines.
+    QString codes;
+    
+    // Create a sequence object we'll use to store the monomers we
+    // have successfully parsed in.
+    Sequence sequence;
+
+    // Iterate in the file and for each line check that it matches
+    // lines containing SEQRES records. If so, process the line to
+    // extract the monomer codes in the style MET LEU THR GLN LYS THR
+
+    // Define a regular expression that will sort out from the file
+    // all the lines that hold a SEQRES record, and specifically one
+    // such record that holds a protein sequence (and not a nucleic
+    // acid sequence), which we know because codes for aminoacids are
+    // 3-letters long in the form of GLY.  "MET LEU THR GLN LYS THR
+    // LYS ASP ILE VAL LYS ALA THR $");
+    
+    QRegExp regExp("^SEQRES "
+		   "[0-9 ][0-9 ][0-9 ] "
+		   "[A-Z ] "
+		   "[0-9 ][0-9 ][0-9 ][0-9 ]  "
+		   "([A-Z][A-Z][A-Z] ){1,13}[ ]*$");
+    
+    QRegExpValidator validator(regExp, 0);
+    int pos = 0;
+
+    QChar currentId;
+    
+    while (!stream.atEnd())
+      {
+	// PDF file format stipulates that:
+
+	// Each line in the PDB entry file consists of 80 columns. The
+	// last character in each PDB entry should be an end-of- line
+	// indicator.
+
+	QString line = stream.readLine(80);
+	
+	// 	qDebug() << __FILE__ << __LINE__
+	// 		 << "line:" << line;
+
+	// Test now if the currently parsed line matches the regular
+	// expression.
+
+	if (validator.validate(line, pos) != QValidator::Acceptable)
+	  {
+	    // 	    qDebug() << __FILE__ << __LINE__
+	    // 		     << "line is faulty:" << line;
+	    
+	    // The line does not match a protein SEQRES record. Go to
+	    // next line in the file.
+	    continue;
+	  }
+	
+	// At this point the line is worth working on: it is a line
+	// describing a portion of a protein sequence. 
+
+	// Note that the file format specification stipulates that
+	// there might be any number of chain sequences. Their ID is
+	// the 'B' at index 11 in the line "SEQRES 29 B 403 TYR TYR
+	// ILE CYS GLY PRO ".
+
+	QChar id = line.at(11);
+
+	if (id != currentId)
+	  {	  
+	    // We are initiating a new chain sequence. Store the
+	    // previously chain sequence into the string list. The
+	    // syntax is that a first string with the chain ID is
+	    // added to the string list and then its sequence.
+  
+	    if (!codes.isEmpty())
+	      {
+		chainStringList->append(codes);
+		chainStringListId->append(QString(currentId));
+
+// 		qDebug() << __FILE__ << __LINE__
+// 			 << "appending codes:"
+// 			 << codes;
+		
+		// Now clear the string so that we can start filling
+		// it with a new chain sequence.
+		codes.clear();
+	      }
+	    
+	    // Now update the current id.
+	    currentId = id;
+	  }
+	
+	//    qDebug() << __FILE__ << __LINE__
+	//     << "line is fine:" << line;
+	
+	
+	// Create a temporary string with the line.
+	QString tempSeq(line);
+	
+	// 	qDebug() << __FILE__ << __LINE__ 
+	// 		 << "tempSeq:" << tempSeq;
+	
+	// From this tempSeq string, only retain the monomer codes:
+	// This is the string:
+	//"SEQRES  29 B  403  TYR TYR ILE CYS GLY PRO "
+	// follows: "ILE PRO PHE MET ARG MET GLN          "
+
+	// This is what we want to retain:
+	// "TYR TYR ILE CYS GLY PRO ILE PRO PHE MET ARG MET GLN"
+	
+	int firstResidueIndex =
+	  tempSeq.indexOf (QRegExp ("([A-Z][A-Z][A-Z] ){1,13}[ ]*$"), 0 );
+
+	int lastResidueIndex = tempSeq.lastIndexOf(QRegExp("[A-Z][A-Z][A-Z]"));
+
+	int stringLength = lastResidueIndex - firstResidueIndex;
+
+	//  	qDebug() << __FILE__ << __LINE__ 
+	//  		 << "firstResidueIndex:" << firstResidueIndex
+	// 		 << "lastResidueIndex:" << lastResidueIndex;
+		
+	QString tempCodes = QString (" %1")
+	  .arg(tempSeq.mid(firstResidueIndex, stringLength + 3));
+	
+	// This is what tempCodes contains now (not the leading
+	// space): " TYR TYR ILE CYS GLY PRO ILE PRO PHE MET ARG MET
+	// GLN"
+
+	// 	qDebug() << __FILE__ << __LINE__ 
+	// 		 << " tempCodes:" <<  tempCodes;
+
+	// Appends the codes from the currently parsed line into the
+	// string to hold all the sequence in the file.
+
+	codes.append(tempCodes);
+      }
+
+    // At this point all the PDB file has been read through. We still
+    // have one ongoing codes string, if not empty which we ought to
+    // add to the chainStringList.
+
+    if (!codes.isEmpty())
+      {
+	chainStringList->append(codes);
+	chainStringListId->append(QString(currentId));
+
+// 	qDebug() << __FILE__ << __LINE__
+// 		 << "appending codes:"
+// 		 << codes;
+	
+	// Clear the codes string to free memory.
+	codes.clear();
+      }    
+   
+    // Now feed the chain string list to the translator. The returned
+    // string list is allocated. We will transfer its ownership to the
+    // dialog, if we use that dialog below.
+    QStringList *translatedChainStringList = 
+      dictionary.translate(*chainStringList);
+    
+    if (!translatedChainStringList)
+      {
+	QMessageBox::critical(this, 
+			      tr("massXpert - Import PDB Protein Sequence"),
+			      tr("Failed to import the sequence(s)."),
+			      QMessageBox::Ok);
+
+	delete chainStringList;
+	delete chainStringListId;
+
+	return;
+      }
+    
+//     for (int iter = 0 ; iter < translatedChainStringList->size(); ++iter)
+//       {
+// 	qDebug() << __FILE__ << __LINE__ 
+// 		 << chainStringListId->at(iter)
+// 		 << translatedChainStringList->at(iter);
+//       }
+
+    // At this point, we will not need the initial list anymore, we
+    // can free it and set it to 0.
+    delete chainStringList;
+    chainStringList = 0;
+
+    // If there is only one sequence that's been imported (the file
+    // format allows for any number of chain sequences), then we have
+    // nothing more than to make an actual sequence with it.
+
+    if (translatedChainStringList->size() == 1)
+      {
+	Sequence sequence(translatedChainStringList->first());
+
+	// Immediately free the string lists and set them to 0 as we
+	// are not going to use them anymore.
+
+	delete chainStringListId;
+	chainStringListId = 0;
+	delete translatedChainStringList;
+	translatedChainStringList = 0;
+
+	// Now test if the sequence is correct.
+	
+	QList<int> errorList ;
+	
+	if (sequence.makeMonomerList(mpa_polymer->polChemDef(),
+				     true, &errorList) == -1)
+	  {
+	    SequencePurificationDlg *dlg = 
+	      new SequencePurificationDlg(this, &sequence, &errorList);
+      
+	    dlg->show();
+      
+	    return;
+	  }
+
+	// At this point we can paste...
+	int size = sequence.monomerList().size();
+	int ret = mpa_editorGraphicsView->insertSequenceAtPoint(sequence);
+  
+	if ( ret != size)
+	  {
+	    QMessageBox::critical(this, 
+				  tr("massXpert - Polymer Sequence Editor"),
+				  tr("Failed to import the sequence."),
+				  QMessageBox::Ok);
+	  }
+
+	return;
+      }
+    
+    // If we are here, then that means that there were more than one
+    // sequence imported from the PDB file. We have to display these
+    // sequences to the user.
+
+    // First, make sure that we have the same number of items in the
+    // chain sequence string list and in the chain id string list.
+
+    if (chainStringListId->size() != translatedChainStringList->size())
+      {
+	QMessageBox::critical(this, 
+			      tr("massXpert - Polymer Sequence Editor"),
+			      tr("Failed to import the sequence:\n"
+				 "Number of chain sequences differs "
+				 "from number of chain IDs."),
+			      QMessageBox::Ok);
+      }
+    
+    // We should open the window that is specialized for displaying
+    // imported sequences to the user. 
+
+    // Note that the translatedChainStringList ownership is transferred to
+    // the dialog. We will not delete it.
+    SequenceImportDlg *dlg = new SequenceImportDlg(this,
+						   translatedChainStringList,
+						   chainStringListId);
+    dlg->show();
+    
+    return;
+  }
+  
 
   void
   SequenceEditorWnd::clipboardCopy(QClipboard::Mode mode)
@@ -2302,10 +2666,10 @@ namespace massXpert
   SequenceEditorWnd::cleave()
   {
     CleavageDlg *dlg = new CleavageDlg(this, 
-					mpa_polymer,
-					mpa_polymer->polChemDef(),
-					&m_calcOptions,
-					&m_ionizeRule);
+				       mpa_polymer,
+				       mpa_polymer->polChemDef(),
+				       m_calcOptions,
+				       &m_ionizeRule);
     dlg->show();
   }
 
