@@ -281,7 +281,7 @@ namespace massXpert
 
   //! Calculates the monoisotopic mass.
   /*! Calculation is done by iterating in the isotope list and looking
-    for the lightest isotope.
+    for the most abundant isotope.
   
     \return the monoisotopic mass.
   */
@@ -289,36 +289,36 @@ namespace massXpert
   Atom::calculateMono()
   {
     double temp = 0;
-    double mass = 0;
-
+    double abund = 0;
+    
     int idx = -1;
 
-    // All we have to do is find what's the isotope that has the lowest
-    // mass.
+    // All we have to do is find what's the isotope that has the
+    // highest abundance.
 
     for (int iter = 0; iter < m_isotopeList.size(); ++iter)
       {
-	mass = m_isotopeList.at(iter)->mass();
+	abund = m_isotopeList.at(iter)->abundance();
 
 	if(temp == 0)
 	  {
-	    temp = mass;
+	    temp = abund;
 	    idx = iter;
 	  }
 	else
 	  {
-	    if (mass < temp)
+	    if (abund > temp)
 	      {
-		temp = mass;
+		temp = abund;
 		idx = iter;
 	      }
 	  }
       }
 
     // At this point, we should have a idx variable greater than or
-    // equal to 0. Note, however, that it might happen that the monomer
-    // has no isotope in its list, for example, when the user is editing
-    // the atom definitions in the atom definition window.
+    // equal to 0. Note, however, that it might happen that the
+    // monomer has no isotope in its list, for example, when the user
+    // is editing the atom definitions in the atom definition window.
     if (idx == -1)
       m_mono = 0;
     else
