@@ -63,7 +63,9 @@ namespace massXpert
     : PolChemDefEntity(polChemDef, name), Formula(formula),
       m_fragEnd(fragEnd), m_comment(comment)
   {
-    m_sideChainContribution = 0;
+    m_fragEnd = MXT_FRAG_END_NONE;
+    
+    m_monomerContribution = 0;
   }
 
 
@@ -77,10 +79,12 @@ namespace massXpert
     \param formula Formula.
   */
   FragSpec::FragSpec(const PolChemDef *polChemDef, QString name, 
-		      QString formula)
+		     QString formula)
     : PolChemDefEntity(polChemDef, name), Formula(formula)
   {
-    m_sideChainContribution = 0;
+    m_fragEnd = MXT_FRAG_END_NONE;
+    
+    m_monomerContribution = 0;
   }
 
 
@@ -90,7 +94,7 @@ namespace massXpert
   FragSpec::FragSpec(const FragSpec &other)
     : PolChemDefEntity(other), Formula(other), 
       m_fragEnd(other.m_fragEnd), 
-      m_sideChainContribution(other.m_sideChainContribution),
+      m_monomerContribution(other.m_monomerContribution),
       m_comment(other.m_comment)
   {
     for (int iter = 0 ; iter < other.m_ruleList.size(); ++iter)
@@ -143,7 +147,7 @@ namespace massXpert
     Formula::clone(other);
 
     other->m_fragEnd = m_fragEnd;
-    other->m_sideChainContribution = m_sideChainContribution;
+    other->m_monomerContribution = m_monomerContribution;
     other->m_comment = m_comment;
   
     while(!other->m_ruleList.isEmpty())
@@ -171,7 +175,7 @@ namespace massXpert
     Formula::mold(other);
 
     m_fragEnd = other.m_fragEnd;
-    m_sideChainContribution = other.m_sideChainContribution;
+    m_monomerContribution = other.m_monomerContribution;
     m_comment = other.m_comment;
   
     while(!m_ruleList.isEmpty())
@@ -279,16 +283,16 @@ namespace massXpert
 
 
   void
-  FragSpec::setSideChainContribution(int value)
+  FragSpec::setMonomerContribution(int value)
   {
-    m_sideChainContribution = value;
+    m_monomerContribution = value;
   }
 
 
   int
-  FragSpec::sideChainContribution()
+  FragSpec::monomerContribution()
   {
-    return m_sideChainContribution;
+    return m_monomerContribution;
   }
 
 
@@ -486,7 +490,7 @@ namespace massXpert
     // The file format does not prescript the <sidechaincontrib>
     // element, but we have to set the corresponding value to 0.
   
-    m_sideChainContribution = 0;
+    m_monomerContribution = 0;
   
     // The next element might be either comment or(none, one or more)
     // fgr.
@@ -622,9 +626,9 @@ namespace massXpert
   
     bool ok = false;
   
-    m_sideChainContribution = text.toInt(&ok);
+    m_monomerContribution = text.toInt(&ok);
   
-    if (!m_sideChainContribution && !ok)
+    if (!m_monomerContribution && !ok)
       return false;
   
     // The next element might be either comment or(none, one or more)
@@ -779,7 +783,7 @@ namespace massXpert
 
     *string += QString("%1<sidechaincontrib>%2</sidechaincontrib>\n")
       .arg(lead)
-      .arg(m_sideChainContribution);
+      .arg(m_monomerContribution);
     
     if (!m_comment.isEmpty())
       *string += QString("%1<comment>%2</comment>\n")
